@@ -39,9 +39,9 @@
 const RDF_NS = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
 const PFS_NS = "http://www.mozilla.org/2004/pfs-rdf#";
 
-function nsRDFItemUpdater(aClientOS, aChromeLocale){
+function nsRDFItemUpdater(aClientOS, aChromeLocale) {
   this._rdfService = Components.classes["@mozilla.org/rdf/rdf-service;1"]
-                        .getService(Components.interfaces.nsIRDFService);
+                               .getService(Components.interfaces.nsIRDFService);
   this._os = Components.classes["@mozilla.org/observer-service;1"]
                        .getService(Components.interfaces.nsIObserverService);
 
@@ -49,6 +49,7 @@ function nsRDFItemUpdater(aClientOS, aChromeLocale){
                       .getService(Components.interfaces.nsIXULAppInfo);
   this.appID = app.ID;
   this.buildID = app.platformBuildID;
+  this.appRelease = app.version;
 
   this.clientOS = aClientOS;
   this.chromeLocale = aChromeLocale;
@@ -56,17 +57,20 @@ function nsRDFItemUpdater(aClientOS, aChromeLocale){
   var prefBranch = Components.classes["@mozilla.org/preferences-service;1"]
                              .getService(Components.interfaces.nsIPrefBranch);
   this.dsURI = prefBranch.getCharPref("pfs.datasource.url");
+  this.distID = prefBranch.getCharPref("extensions.ubufox@ubuntu.com.release");
 }
 
 nsRDFItemUpdater.prototype = {
-  checkForPlugin: function (aPluginRequestItem){
+  checkForPlugin: function (aPluginRequestItem) {
     var dsURI = this.dsURI;
     // escape the mimetype as mimetypes can contain '+', which will break pfs.
     dsURI = dsURI.replace(/%PLUGIN_MIMETYPE%/g, encodeURIComponent(aPluginRequestItem.mimetype));
     dsURI = dsURI.replace(/%APP_ID%/g, this.appID);
     dsURI = dsURI.replace(/%APP_VERSION%/g, this.buildID);
+    dsURI = dsURI.replace(/%APP_RELEASE%/g, this.appRelease);
     dsURI = dsURI.replace(/%CLIENT_OS%/g, this.clientOS);
     dsURI = dsURI.replace(/%CHROME_LOCALE%/g, this.chromeLocale);
+    dsURI = dsURI.replace(/%DIST_ID%/g, this.distID);
 
     var ds = this._rdfService.GetDataSource(dsURI);
     var rds = ds.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource)
@@ -78,14 +82,13 @@ nsRDFItemUpdater.prototype = {
     }
   },
 
-  onDatasourceLoaded: function pfs_onDatasourceLoaded (aDatasource, aPluginRequestItem){
-    var container = Components.classes["@mozilla.org/rdf/container;1"].
-                  createInstance(Components.interfaces.nsIRDFContainer);
+  onDatasourceLoaded: function pfs_onDatasourceLoaded (aDatasource, aPluginRequestItem) {
+    var container = Components.classes["@mozilla.org/rdf/container;1"]
+                              .createInstance(Components.interfaces.nsIRDFContainer);
     var resultRes = this._rdfService.GetResource("urn:mozilla:plugin-results:" + aPluginRequestItem.mimetype);
     var pluginList = aDatasource.GetTarget(resultRes, this._rdfService.GetResource(PFS_NS+"plugins"), true);
     var pluginInfos = new Array();
-  
-    container = Components.classes["@mozilla.org/rdf/container;1"].createInstance(Components.interfaces.nsIRDFContainer);
+
     try {
       container.Init(aDatasource, pluginList);
 
@@ -95,7 +98,7 @@ nsRDFItemUpdater.prototype = {
 
 	var target;
 	var child = children.getNext();
-	if (child instanceof Components.interfaces.nsIRDFResource){
+	if (child instanceof Components.interfaces.nsIRDFResource) {
 	  var name = this._rdfService.GetResource("http://www.mozilla.org/2004/pfs-rdf#updates");
 	  target = aDatasource.GetTarget(child, name, true);
 	}
@@ -107,13 +110,13 @@ nsRDFItemUpdater.prototype = {
 
 	  while (children2.hasMoreElements()) {
 	    var child2 = children2.getNext();
-	    if (child2 instanceof Components.interfaces.nsIRDFResource){
+	    if (child2 instanceof Components.interfaces.nsIRDFResource) {
 	      target2 = child2;
 	    }
 
 	    var rdfs = this._rdfService;
 
-	    function getPFSValueFromRDF(aValue){
+	    function getPFSValueFromRDF(aValue) {
 	      var rv = null;
 
 	      var myTarget = aDatasource.GetTarget(target2, rdfs.GetResource(PFS_NS + aValue), true);
@@ -128,33 +131,45 @@ nsRDFItemUpdater.prototype = {
 	      pid: getPFSValueFromRDF("guid"),
 	      version: getPFSValueFromRDF("version"),
 	      IconUrl: getPFSValueFromRDF("IconUrl"),
+              InstallerLocation: getPFSValueFromRDF("InstallerLocation"),
+	      desc: getPFSValueFromRDF("description"), 
+	      homepage: getPFSValueFromRDF("homepage"),
 	      XPILocation: getPFSValueFromRDF("XPILocation"),
 	      XPIHash: getPFSValueFromRDF("XPIHash"),
 	      InstallerShowsUI: getPFSValueFromRDF("InstallerShowsUI"),
 	      manualInstallationURL: getPFSValueFromRDF("manualInstallationURL"),
 	      requestedMimetype: getPFSValueFromRDF("requestedMimetype"),
 	      licenseURL: getPFSValueFromRDF("licenseURL"),
-	      needsRestart: getPFSValueFromRDF("needsRestart")
+	      needsRestart: getPFSValueFromRDF("needsRestart"),
+	      fileHint: getPFSValueFromRDF("filehint")
 	    };
 
+        // no license provided, make it accepted
+        pluginInfo.licenseAccepted = pluginInfo.licenseURL ? false : true;
+
 	    pluginInfos.push(pluginInfo);
 	  }
 	}
-	catch (ex){}
+	catch (ex) {
+          Components.utils.reportError(ex);
+        }
       }
     }
-    catch (ex){}
+    catch (ex) {
+      Components.utils.reportError(ex);
+    }
     
-    gPluginInstaller.pluginInfoReceived(pluginInfos);
+    gPluginInstaller.pluginInfoReceived(aPluginRequestItem, pluginInfos);
   },
 
-  onDatasourceError: function pfs_onDatasourceError (aPluginRequestItem, aError){
+  onDatasourceError: function pfs_onDatasourceError (aPluginRequestItem, aError) {
     this._os.notifyObservers(aPluginRequestItem, "error", aError);
-    gPluginInstaller.pluginInfoReceived(null);
-  },
+    Components.utils.reportError(aError);
+    gPluginInstaller.pluginInfoReceived(aPluginRequestItem, null);
+  }
 };
 
-function nsPluginXMLRDFDSObserver(aUpdater, aPluginRequestItem){
+function nsPluginXMLRDFDSObserver(aUpdater, aPluginRequestItem) {
   this._updater = aUpdater;
   this._item    = aPluginRequestItem;
 }
@@ -165,17 +180,17 @@ nsPluginXMLRDFDSObserver.prototype =
   _item     : null,
 
   // nsIRDFXMLSinkObserver
-  onBeginLoad: function(aSink){},
-  onInterrupt: function(aSink){},
-  onResume: function(aSink){},
-  onEndLoad: function(aSink){
+  onBeginLoad: function(aSink) {},
+  onInterrupt: function(aSink) {},
+  onResume: function(aSink) {},
+  onEndLoad: function(aSink) {
     aSink.removeXMLSinkObserver(this);
     
     var ds = aSink.QueryInterface(Components.interfaces.nsIRDFDataSource);
     this._updater.onDatasourceLoaded(ds, this._item);
   },
   
-  onError: function(aSink, aStatus, aErrorMsg){  
+  onError: function(aSink, aStatus, aErrorMsg) {  
     aSink.removeXMLSinkObserver(this);   
     this._updater.onDatasourceError(this._item, aStatus.toString());
   }
