@@ -64,14 +64,15 @@ function nsPluginInstallerWizard(){
   // how many plugins are to be installed
   this.pluginsToInstallNum = 0;
 
-  this.mTab = null;
   this.mBrowser = null;
   this.mSuccessfullPluginInstallation = 0;
+  this.mNeedsRestart = false;
 
   this.mPluginPidArray = new Object();
   // arguments[0] is an array that contains two items:
   //     an array of mimetypes that are missing
-  //     a reference to the tab that needs them, so we can reload it
+  //     a reference to the browser that needs them, 
+  //        so we can notify which browser can be reloaded.
 
   if ("arguments" in window) {
     for (var item in window.arguments[0].plugins){
@@ -81,8 +82,7 @@ function nsPluginInstallerWizard(){
       this.mPluginRequestArrayLength++;
     }
 
-    this.mBrowser = window.arguments[0].browser; // ffox 3
-    this.mTab = window.arguments[0].tab; // ffox 2
+    this.mBrowser = window.arguments[0].browser;
   }
 
   this.WSPluginCounter = 0;
@@ -100,14 +100,14 @@ nsPluginInstallerWizard.prototype.getPluginData = function (){
   // initiate the datasource call
   var rdfUpdater = new nsRDFItemUpdater(this.getOS(), this.getChromeLocale());
 
-  for (item in this.mPluginRequestArray) {
+  for (var item in this.mPluginRequestArray) {
     rdfUpdater.checkForPlugin(this.mPluginRequestArray[item]);
   }
 }
 
 // aPluginInfo is null if the datasource call failed, and pid is -1 if
 // no matching plugin was found.
-nsPluginInstallerWizard.prototype.pluginInfoReceived = function (aPluginInfos){
+nsPluginInstallerWizard.prototype.pluginInfoReceived = function (aPluginRequestItem, aPluginInfos){
   this.WSPluginCounter++;
 
   if (aPluginInfos ) {
@@ -121,7 +121,7 @@ nsPluginInstallerWizard.prototype.pluginInfoReceived = function (aPluginInfos){
       aPluginInfo = aPluginInfos[i];
       resultSetMimeType = aPluginInfo.requestedMimetype;
       if (aPluginInfo && aPluginInfo.pid != -1 &&
-	  ( aPluginInfo.XPILocation || aPluginInfo.manualInstallationURL )) {
+	  ( aPluginInfo.XPILocation || aPluginInfo.manualInstallationURL || aPluginInfo.InstallerLocation)) {
 	hasResults = true;
 	filteredPluginInfoSet.push(aPluginInfo);
 	this.mPluginPidArray[aPluginInfo.pid] = aPluginInfo;
@@ -130,10 +130,10 @@ nsPluginInstallerWizard.prototype.pluginInfoReceived = function (aPluginInfos){
 
     if(filteredPluginInfoSet.length > 0)
     {
-      this.mPluginInfoArray[resultSetMimeType] = filteredPluginInfoSet;
+      this.mPluginInfoArray[aPluginRequestItem.mimetype] = filteredPluginInfoSet;
       this.mPluginInfoArrayLength++;
     } else {
-      this.mPluginNotFoundArray[resultSetMimeType] = filteredPluginInfoSet;
+      this.mPluginNotFoundArray[aPluginRequestItem.mimetype] = aPluginRequestItem;
       this.mPluginNotFoundArrayLength++;
     }
   }
@@ -148,24 +148,25 @@ nsPluginInstallerWizard.prototype.pluginInfoReceived = function (aPluginInfos){
 
   if (this.WSPluginCounter == this.mPluginRequestArrayLength) {
     // check if no plugins were found
-    if (this.mPluginInfoArrayLength == 0 && this.mPluginInfoAptArrayLength == 0) {
-      this.advancePage("lastpage", true, false, false);
+    if (this.mPluginInfoArrayLength == 0) {
+      this.advancePage("lastpage");
     } else {
       // we want to allow user to cancel
-      this.advancePage(null, true, false, true);
+      this.advancePage(null);
     }
   } else {
     // process more.
   }
 }
 
-nsPluginInstallerWizard.prototype.createPluginSetGroupBox = function () {
+nsPluginInstallerWizard.prototype.createPluginSetGroupBox = function (mimetype) {
 
+  var stringBundle = document.getElementById("ubufoxPluginWizardString");
   var gbox = document.createElement("vbox");
   gbox.setAttribute("flex", "1");
 
   var caption = document.createElement("caption");
-  caption.setAttribute("label", this.getString("ubufox.pluginWizard.availablePluginsPage.description.label")+" "+mimetype+":");
+  caption.setAttribute("label", stringBundle.getString("ubufox.pluginWizard.availablePluginsPage.description.label")+" "+mimetype+":");
   gbox.appendChild(caption);
 
   return gbox;
@@ -192,11 +193,11 @@ nsPluginInstallerWizard.prototype.showPluginList = function () {
   var groupBox = null;
   var lastSibling = null;
 
-  for (mimetype in this.mPluginInfoArray){
+  for (var mimetype in this.mPluginInfoArray){
     // [plugin image] [Plugin_Name Plugin_Version]
     var pluginInfoSet = this.mPluginInfoArray[mimetype];
     var firstPluginSet = (groupBox == null);
-    groupBox = this.createPluginSetGroupBox(document, mimetype, pluginInfoSet);
+    groupBox = this.createPluginSetGroupBox(mimetype);
 
     if(firstPluginSet) {
       this.mPluginPlaceHolder = document.getElementById("pluginselection-placeholder");
@@ -274,13 +275,17 @@ nsPluginInstallerWizard.prototype.showPluginList = function () {
     if (pluginInfo.InstallerShowsUI == "true")
       hasPluginWithInstallerUI = true;
 
+    // keep a running count of plugins the user wants to install
     this.pluginsToInstallNum++;
   }
 
   if (hasPluginWithInstallerUI)
     document.getElementById("installerUI").hidden = false;
 
-  this.canAdvance(false);
+  if (this.pluginsToInstallNum > 0)
+    this.canAdvance(true);
+  else
+    this.canAdvance(false);
   this.canRewind(false);
 }
 
@@ -289,14 +294,14 @@ nsPluginInstallerWizard.prototype.toggleInstallPlugin = function (e) {
   var mime = selectedItem._ubufoxPluginInfoMimeType;
   this.mMimeTypePluginSelections[mime] = selectedItem._ubufoxPluginInfo.pid;
 
-  let count  = 0;
-  for (mime in this.mMimeTypePluginSelections) {
+  this.pluginsToInstallNum = 0;
+  for (var mime in this.mMimeTypePluginSelections) {
     if(this.mMimeTypePluginSelections[mime] && this.mMimeTypePluginSelections[mime] != "-1")
-      count++;
+      this.pluginsToInstallNum++;
   }
  
   // if no plugins are checked, don't allow to advance
-  if (count > 0)
+  if (this.pluginsToInstallNum > 0)
     this.canAdvance(true);
   else
     this.canAdvance(false);
@@ -320,7 +325,7 @@ nsPluginInstallerWizard.prototype.showLicenses = function (){
 
   // only add if a license is provided and the plugin was selected to
   // be installed
-  for (mimetype in this.mMimeTypePluginSelections){
+  for (var mimetype in this.mMimeTypePluginSelections){
     var pid = this.mMimeTypePluginSelections[mimetype];
     var pluginInfo = this.mPluginPidArray[pid];
     if (pluginInfo && pluginInfo.licenseURL && (pluginInfo.licenseURL != "")) {
@@ -330,7 +335,7 @@ nsPluginInstallerWizard.prototype.showLicenses = function (){
 
   if (this.mPluginLicenseArray.length == 0) {
     // no plugins require licenses
-    this.advancePage(null, true, false, false);
+    this.advancePage(null);
   } else {
     this.licenseAcceptCounter = 0;
 
@@ -388,7 +393,7 @@ nsPluginInstallerWizard.prototype.showLicense = function (){
 
   this.canAdvance(false);
 
-  loadFlags = Components.interfaces.nsIWebNavigation.LOAD_FLAGS_NONE;
+  var loadFlags = Components.interfaces.nsIWebNavigation.LOAD_FLAGS_NONE;
 
   document.getElementById("licenseIFrame").webNavigation.loadURI(pluginInfo.licenseURL, loadFlags, null, null, null);
 
@@ -446,60 +451,46 @@ nsPluginInstallerWizard.prototype.licenseRadioGroupChange = function(aAccepted)
     this.canAdvance(aAccepted);
 }
 
-nsPluginInstallerWizard.prototype.advancePage = function (aPageId, aCanAdvance, aCanRewind, aCanCancel){
+nsPluginInstallerWizard.prototype.advancePage = function (aPageId){
   this.canAdvance(true);
   document.getElementById("plugin-installer-wizard").advance(aPageId);
-
-  this.canAdvance(aCanAdvance);
-  this.canRewind(aCanRewind);
-  this.canCancel(aCanCancel);
 }
 
 nsPluginInstallerWizard.prototype.startPluginInstallation = function (){
   this.canAdvance(false);
   this.canRewind(false);
 
-  // since the user can choose what plugins to install, we need to store
-  // which ones were choosen, as nsIXPInstallManager returns an index and not the
-  // mimetype.  So store the pids.
-
-  // for ubutfox we deal with multiple cases: first case is XPIInstall, which will
+  // for ubufox we deal with multiple cases: first case is XPIInstall, which will
   // run the "normal" XPIInstall process; second case is XPIInstall url contains
   // and apt: protocol url ... this will run apt protocol handler 
-  var pluginURLArray = new Array();
-  var pluginHashArray = new Array();
-  var pluginPidArray = new Array();
-  this.mAptPluginURLArray = new Array();
-  this.mAptPluginPidArray = new Array();
+  var installerPlugins = [];
+  var xpiPlugins = [];
+  this.aptPlugins = [];
 
-  for (mime in this.mMimeTypePluginSelections) {
-    var pluginPid = this.mMimeTypePluginSelections[mime];
-    var pluginItem = this.mPluginPidArray[pluginPid];
+  for (var mime in this.mMimeTypePluginSelections) {
+    var pluginItem = this.mPluginPidArray[this.mMimeTypePluginSelections[mime]];
 
-    // only push to the array if it has an XPILocation, else nsIXPInstallManager
-    // will complain.
-    if (pluginItem && pluginItem.XPILocation && pluginItem.XPILocation.indexOf("apt:") != 0 && pluginItem.licenseAccepted) {
-      pluginURLArray.push(pluginItem.XPILocation);
-      pluginHashArray.push(pluginItem.XPIHash);
-      pluginPidArray.push(pluginItem.pid);
-    } else if (pluginItem && pluginItem.XPILocation && pluginItem.XPILocation.indexOf("apt:") == 0) {
-      this.mAptPluginURLArray.push(pluginItem.XPILocation);
-      this.mAptPluginPidArray.push(pluginPid);
-    } else {
-      window.alert("Unhandled mime install flavour (supported: vendor, apt)");
-      continue;
+    if (pluginItem && pluginItem.licenseAccepted) {
+      if (pluginItem.InstallerLocation)
+        installerPlugins.push(pluginItem);
+      else if (pluginItem.XPILocation && pluginItem.XPILocation.indexOf("apt:") != 0)
+        xpiPlugins.push(pluginItem);
+      else if (pluginItem.XPILocation && pluginItem.XPILocation.indexOf("apt:") == 0)
+        this.aptPlugins.push(pluginItem);
+      else {
+        window.alert("Unhandled mime install flavour (supported: vendor, apt)");
+        continue;
+      }
     }
   }
 
-  if (pluginURLArray.length > 0)
-    PluginXPIInstallService.startPluginInstallation(pluginURLArray,
-						    pluginHashArray,
-						    pluginPidArray);
-  else if (this.mAptPluginURLArray.length > 0)
-    PluginAPTInstallService.startPluginInstallation(this.mAptPluginURLArray,
-						    this.mAptPluginPidArray);
+  if (installerPlugins.length > 0 || xpiPlugins.length > 0)
+    PluginXPIInstallService.startPluginInstallation(installerPlugins,
+						    xpiPlugins);
+  else if (this.aptPlugins.length > 0)
+    PluginAPTInstallService.startPluginInstallation(this.aptPlugins);
   else
-    this.advancePage(null, true, false, false);
+    this.advancePage(null);
 }
 
 /*
@@ -529,6 +520,8 @@ nsPluginInstallerWizard.prototype.pluginXPIInstallationProgress = function (aPid
 
     case 2:
       statMsg = this.getFormattedString("pluginInstallation.install.start", [pluginInfo.name]);
+      var progressElm = document.getElementById("plugin_install_progress");
+      progressElm.setAttribute("mode", "undetermined");
       break;
     case 6:
       statMsg = "APT - " + this.getFormattedString("pluginInstallation.install.start", [pluginInfo.name]);
@@ -554,11 +547,10 @@ nsPluginInstallerWizard.prototype.pluginXPIInstallationProgress = function (aPid
       break;
 
     case 4:
-      PluginAPTInstallService.startPluginInstallation(this.mAptPluginURLArray,
-						      this.mAptPluginPidArray);
+      PluginAPTInstallService.startPluginInstallation(this.aptPlugins);
       break;
     case 8:
-      this.advancePage(null, true, false, false);
+      this.advancePage(null);
       statMsg = this.getString("pluginInstallation.complete");
       break;
     default:
@@ -629,15 +621,12 @@ nsPluginInstallerWizard.prototype.showPluginResults = function (){
   var notInstalledList = "?action=missingplugins";
   var myRows = document.getElementById("pluginResultList");
 
-  this.mNeedsRestart = false;
-
   // clear children
   for (var run = myRows.childNodes.length; run--; run > 0)
     myRows.removeChild(myRows.childNodes.item(run));
 
-  for (mimetype in this.mMimeTypePluginSelections) {
-    var pid = this.mMimeTypePluginSelections[mimetype];
-    var pluginInfoItem = this.mPluginPidArray[pid];
+  for (var mimetype in this.mMimeTypePluginSelections) {
+    var pluginInfoItem = this.mPluginPidArray[this.mMimeTypePluginSelections[mimetype]];
     // [plugin image] [Plugin_Name Plugin_Version] [Success/Failed] [Manual Install (if Failed)]
 
     var myPluginItem = pluginInfoItem; //this.mPluginInfoArray[pluginInfoItem];
@@ -650,7 +639,7 @@ nsPluginInstallerWizard.prototype.showPluginResults = function (){
       notInstalledList += "&mimetype=" + pluginInfoItem;
     } else if (myPluginItem.licenseURL && !myPluginItem.licenseAccepted) {
       statusMsg = this.getString("pluginInstallationSummary.licenseNotAccepted");
-    } else if (!myPluginItem.XPILocation) {
+    } else if (!myPluginItem.XPILocation && !myPluginItem.InstallerLocation) {
       statusMsg = this.getString("pluginInstallationSummary.notAvailable");
       notInstalledList += "&mimetype=" + pluginInfoItem;
     } else {
@@ -659,12 +648,12 @@ nsPluginInstallerWizard.prototype.showPluginResults = function (){
 
       // only check needsRestart if the plugin was successfully installed.
       if (myPluginItem.needsRestart)
-	this.mNeedsRestart = false;
+	this.mNeedsRestart = true;
     }
 
     // manual url - either returned from the webservice or the pluginspage attribute
     var manualUrl;
-    if ((myPluginItem.error || !myPluginItem.XPILocation) && (myPluginItem.manualInstallationURL || this.mPluginRequestArray[myPluginItem.requestedMimetype].pluginsPage)){
+    if ((myPluginItem.error || (!myPluginItem.XPILocation && !myPluginItem.InstallerLocation)) && (myPluginItem.manualInstallationURL || this.mPluginRequestArray[myPluginItem.requestedMimetype].pluginsPage)){
       manualUrl = myPluginItem.manualInstallationURL ? myPluginItem.manualInstallationURL : this.mPluginRequestArray[myPluginItem.requestedMimetype].pluginsPage;
     }
 
@@ -679,10 +668,10 @@ nsPluginInstallerWizard.prototype.showPluginResults = function (){
 
   // handle plugins we couldn't find
   for (pluginInfoItem in this.mPluginNotFoundArray){
-    var pluginRequest = this.mPluginRequestArray[pluginInfoItem];
+    var pluginRequest = this.mPluginNotFoundArray[pluginInfoItem];
 
     // if there is a pluginspage, show UI
-    if (pluginRequest) {
+    if (pluginRequest.pluginsPage) {
       this.addPluginResultRow(
           "",
           this.getFormattedString("pluginInstallation.unknownPlugin", [pluginInfoItem]),
@@ -715,28 +704,43 @@ nsPluginInstallerWizard.prototype.showPluginResults = function (){
     "&appID=" + app.ID +
     "&appVersion=" + app.platformBuildID +
     "&clientOS=" + this.getOS() +
-    "&chromeLocale=" + this.getChromeLocale();
+    "&chromeLocale=" + this.getChromeLocale() +
+    "&appRelease=" + app.version;
 
   document.getElementById("moreInfoLink").addEventListener("click", function() { gPluginInstaller.loadURL("https://pfs.mozilla.org/plugins/" + notInstalledList) }, false);
 
+  if (this.mNeedsRestart) {
+    var cancel = document.getElementById("plugin-installer-wizard").getButton("cancel");
+    cancel.label = this.getString("pluginInstallation.close.label");
+    cancel.accessKey = this.getString("pluginInstallation.close.accesskey");
+    var finish = document.getElementById("plugin-installer-wizard").getButton("finish");
+    finish.label = this.getFormattedString("pluginInstallation.restart.label", [app.name]);
+    finish.accessKey = this.getString("pluginInstallation.restart.accesskey");
+    this.canCancel(true);
+  }
+  else {
+    this.canCancel(false);
+  }
+
   this.canAdvance(true);
   this.canRewind(false);
-  this.canCancel(false);
 }
 
 nsPluginInstallerWizard.prototype.loadURL = function (aUrl){
   // Check if the page where the plugin came from can load aUrl before
-  // loading it, and do *not* allow loading javascript: or data: URIs.
-  var pluginPage = window.opener.content.location.href;
+  // loading it, and do *not* allow loading URIs that would inherit our
+  // principal.
+  
+  var pluginPagePrincipal =
+    window.opener.content.document.nodePrincipal;
 
   const nsIScriptSecurityManager =
     Components.interfaces.nsIScriptSecurityManager;
-  var secMan =
-    Components.classes["@mozilla.org/scriptsecuritymanager;1"]
-    .getService(nsIScriptSecurityManager);
+  var secMan = Components.classes["@mozilla.org/scriptsecuritymanager;1"]
+                         .getService(nsIScriptSecurityManager);
 
-  secMan.checkLoadURIStr(pluginPage, aUrl,
-                         nsIScriptSecurityManager.DISALLOW_SCRIPT_OR_DATA);
+  secMan.checkLoadURIStrWithPrincipal(pluginPagePrincipal, aUrl,
+    nsIScriptSecurityManager.DISALLOW_INHERIT_PRINCIPAL);
 
   window.opener.open(aUrl);
 }
@@ -791,6 +795,8 @@ function PluginInfo(aResult) {
   this.pid = aResult.pid;
   this.version = aResult.version;
   this.IconUrl = aResult.IconUrl;
+  this.InstallerLocation = aResult.InstallerLocation;
+  this.InstallerHash = aResult.InstallerHash;
   this.XPILocation = aResult.XPILocation;
   this.XPIHash = aResult.XPIHash;
   this.InstallerShowsUI = aResult.InstallerShowsUI;
@@ -815,7 +821,6 @@ function wizardInit(){
 }
 
 function wizardFinish(){
-  // we restart if we have no choice ...
   if (gPluginInstaller.mNeedsRestart) {
     // Notify all windows that an application quit has been requested.
     var os = Components.classes["@mozilla.org/observer-service;1"]
@@ -834,21 +839,26 @@ function wizardFinish(){
     }
   }
 
-  if (gPluginInstaller.mBrowser) { // ffox 3 code can autoscan ...
-    // always refresh
-    var event = document.createEvent("Events");
-    event.initEvent("NewPluginInstalled", true, true);
-    var dispatched = gPluginInstaller.mBrowser.dispatchEvent(event);
-  }
-  else if (gPluginInstaller.mTab) { // ffox 2 code can autoscan ...
-    if ((gPluginInstaller.mSuccessfullPluginInstallation > 0) &&
-       (gPluginInstaller.mPluginInfoArrayLength != 0)) {
-      // clear the tab's plugin list only if we installed at least one plugin
-      gPluginInstaller.mTab.missingPlugins = null;
-      // reset UI
-      window.opener.gMissingPluginInstaller.closeNotification();
-      // reload the browser to make the new plugin show
-      window.opener.getBrowser().reloadTab(gPluginInstaller.mTab);
+  // don't refresh if no plugins were found or installed
+  if ((gPluginInstaller.mSuccessfullPluginInstallation > 0) &&
+      (gPluginInstaller.mPluginInfoArray.length != 0)) {
+
+    // reload plugins so JS detection works immediately
+    try {
+      var ph = Components.classes["@mozilla.org/plugin/host;1"]
+                         .getService(Components.interfaces.nsIPluginHost);
+      ph.reloadPlugins(false);
+    }
+    catch (e) {
+      // reloadPlugins throws an exception if there were no plugins to load
+    }
+
+    if (gPluginInstaller.mBrowser) {
+      // notify listeners that a plugin is installed,
+      // so that they can reset the UI and update the browser.
+      var event = document.createEvent("Events");
+      event.initEvent("NewPluginInstalled", true, true);
+      gPluginInstaller.mBrowser.dispatchEvent(event);
     }
   }
 
