@@ -32,7 +32,7 @@ archs=sys.argv[2]
 
 print "RELEASES", releases
 
-mimetype_pkgweights = { "application/x-shockwave-flash" : { "flashplugin-nonfree" : 9 } }	
+mimetype_pkgweights = { "application/x-shockwave-flash" : { "flashplugin-nonfree" : 9, "flashplugin-installer" : 9 } }	
 
 class AptPluginDbUpdater:
 
